import unittest as ut
import adios as ad
import numpy as np
from common import TempFile
from common import Slicee

class AdiosTestCase(ut.TestCase):
    f    = None ## Adios File class
    temp = None ## TempFile

    def setUp(self):
        ad.init_noxml()

    def tearDown(self):
        ad.finalize()

    def test_writer_var(self):
        self.temp = TempFile()

        NX = 10
        val1 = np.array(list(range(NX)), dtype=np.int32)
        val2 = np.array(list(range(5)), dtype='f8')

        fw = ad.writer(self.temp.path)
        fw.declare_group("group", method="POSIX1")

        fw.define_var("NX")
        fw.define_var("val1", "NX")
        fw.define_var("val2", val2.shape)

        fw['NX'] = NX
        fw['val1'] = val1
        fw['val2'] = val2
        fw.close()

        f = ad.file(self.temp.path)
        self.assertEqual(f['NX'][...], NX)
        self.assertTrue((f['val1'][:] == val1).all())
        self.assertTrue((f['val2'][:] == val2).all())

    def test_writer_attr(self):
        self.temp = TempFile()

        NX = 10
        val1 = np.array(list(range(NX)), dtype=np.int32)
        val2 = np.array(list(range(5)), dtype='f8')

        single_string = "ABCD"
        three_string = ("AA","BBB","CCCC")
        single_int = 10
        five_int = np.array(list(range(5)), dtype=np.int32)
        single_double = 1.1
        five_double = np.array(list(range(5)), dtype='double')*1.1
        unicode_string = u"unicode"
        bytes_string = u"bytes"

        fw = ad.writer(self.temp.path)
        fw.declare_group("group", method="POSIX1")

        fw.define_attr("single_string")
        fw.define_attr("three_string")
        fw.define_attr("single_int")
        fw.define_attr("five_int")
        fw.define_attr("single_double")
        fw.define_attr("five_double")
        fw.define_attr("unicode_string")
        fw.define_attr("bytes_string")

        fw['single_string'] = single_string
        fw['three_string'] = three_string
        fw['single_int'] = single_int
        fw['five_int'] = five_int
        fw['single_double'] = single_double
        fw['five_double'] = five_double
        fw['unicode_string'] = unicode_string
        fw['bytes_string'] = bytes_string
        fw.close()

        f = ad.file(self.temp.path)
        self.assertEqual(f['single_string'].value, single_string.encode())
        ##self.assertTrue((f['three_string'].value == three_string).all())
        self.assertTrue(f['three_string'].value[0], three_string[0].encode())
        self.assertTrue(f['three_string'].value[1], three_string[1].encode())
        self.assertTrue(f['three_string'].value[2], three_string[2].encode())
        self.assertEqual(f['single_int'].value, single_int)
        self.assertTrue((f['five_int'].value == five_int).all())
        self.assertEqual(f['single_double'].value, single_double)
        self.assertTrue((f['five_double'].value == five_double).all())
        self.assertTrue(f['unicode_string'].value, unicode_string.encode())
        self.assertTrue(f['bytes_string'].value, bytes_string)

    def test_writer_undefined_var(self):
        self.temp = TempFile()

        NX = 10
        val1 = np.array(list(range(NX)), dtype=np.int32)
        val2 = np.array(list(range(5)), dtype='f8')

        fw = ad.writer(self.temp.path)
        fw.declare_group("group", method="POSIX1")

        fw['NX'] = NX
        fw['val1'] = val1
        fw['val2'] = val2
        fw.close()

        f = ad.file(self.temp.path)
        self.assertEqual(f['NX'][...], NX)
        self.assertTrue((f['val1'][:] == val1).all())
        self.assertTrue((f['val2'][:] == val2).all())

    def test_writer_undefined_var2(self):
        self.temp = TempFile()

        NX = 10
        val1 = np.array(list(range(NX)), dtype=np.int32)
        val2 = np.array(list(range(5)), dtype='f8')

        fw = ad.writer(self.temp.path)
        fw.declare_group("group", method="POSIX1")

        fw.vars['NX'] = NX
        fw.vars['val1'] = val1
        fw.vars['val2'] = val2
        fw.close()

        f = ad.file(self.temp.path)
        self.assertEqual(f['NX'][...], NX)
        self.assertTrue((f['val1'][:] == val1).all())
        self.assertTrue((f['val2'][:] == val2).all())

    def test_writer_default_group(self):
        self.temp = TempFile()
        fw = ad.writer(self.temp.path)
        fw.close()

        f = ad.file(self.temp.path)
        f.close()

    def test_writer_varname(self):
        self.temp = TempFile()

        fw = ad.writer(self.temp.path)
        fw.declare_group("group", method="POSIX1")

        NVARS = 99
        fw.vars['nvars'] = NVARS
        fw.vars['/nvars'] = NVARS
        fw.vars['_nvars'] = NVARS
        fw.close()

        f = ad.file(self.temp.path)
        self.assertEqual(f.nvars, 3)
        self.assertEqual(f._nvars[...], NVARS)

if __name__ == '__main__':
    ut.main()



import re

def join(*args):
    tokens = []
    for tok in args:
        if isinstance(tok, (list, tuple)):
            tok = '(%s)' % r'\s*'.join(tok)
        tokens.append(tok)
    return r'\s*'.join(tokens)

lparen   = r'\('
rparen   = r'\)'
colon    = r'\:'
asterisk = r'\*'
eol      = r'$'

enum    = join('enum', colon)
typedef = 'ctypedef'
pointer = asterisk
struct  = join(typedef, 'struct')

basic_type    = r'(?:void|int|char\s*\*{1,3})'
struct_type   = r'MPI_(?:Status)'
integral_type = r'MPI_(?:Aint|Offset)'
opaque_type   = r'MPI_(?:Datatype|Request|Op|Info|Group|Errhandler|Comm|Win|File)'
any_mpi_type  = r'(?:%s|%s|%s)' % (struct_type, integral_type, opaque_type)

upper_name  = r'MPI_[A-Z0-9_]+'
camel_name  = r'MPI_[A-Z][a-z0-9_]+'
usrfun_name = camel_name + r'_(?:function|fn)'

arg_list = r'.*'
ret_type = r'void|int|double'


canylong = join(r'long', r'(?:long)?')
canyptr  = join(r'\w+', pointer+'?')

annotation = r'\#\:\='
defval = r'(?:%s)?' % join (annotation, [r'\(?[A-Za-z0-9_\+\-\(\)\*]+\)?'])

STRUCT_TYPE   = join( struct,  [struct_type] , colon,  eol)
INTEGRAL_TYPE = join( typedef, canylong, [integral_type], eol)
OPAQUE_TYPE   = join( typedef, canyptr,  [opaque_type],   eol)
FUNCTION_TYPE = join( typedef, [ret_type], [camel_name],
                      lparen, [arg_list], rparen,
                      defval, eol)

ENUM_VALUE     = join( enum,          [upper_name], defval, eol)
HANDLE_VALUE   = join( [opaque_type], [upper_name], defval, eol)
BASICP_VALUE   = join( [basic_type,  pointer], [upper_name], defval , eol)
STRUCTP_VALUE  = join( [struct_type, pointer], [upper_name], defval , eol)
FUNCTP_VALUE   = join( [usrfun_name, pointer], [upper_name], defval , eol)
FUNCTION_PROTO = join([ret_type], [camel_name],
                      lparen, [arg_list], rparen,
                      defval, eol)

fint_type = r'MPI_Fint'
c2f_name  = r'MPI_[A-Z][a-z_]+_c2f'
f2c_name  = r'MPI_[A-Z][a-z_]+_f2c'

FINT_TYPE    = join( typedef, canylong, [fint_type], eol)
FINTP_VALUE  = join( [fint_type, pointer], [upper_name], defval , eol)
FUNCTION_C2F = join([fint_type], [c2f_name],
                    lparen, [opaque_type], rparen,
                    defval, eol)
FUNCTION_F2C = join([opaque_type], [f2c_name],
                    lparen, [fint_type], rparen,
                    defval, eol)


# compile the RE's
glb = globals()
all = [key for key in dict(glb) if key.isupper()]
for key in all: glb[key] = re.compile(glb[key])



#!/usr/bin/env python
# Author:  Jong Choi
# Contact: choij@ornl.gov


import os
import sys
import argparse

## Credit: http://svn.apache.org/repos/asf/subversion/tags/0.20.1/subversion/bindings/swig/python/setup.py
# def _do_usage():
#   print "Usage: setup.py [OPTIONS] build"
#   print "       setup.py install [--prefix PREFIX]"
#   print "       setup.py install_lib [--install-dir DIR]"
#   print ""
#   print "Options:"
#   print "   -I dir      " + \
#         "search DIR for includes (multiple instances allowed)"
#   print "   -L dir      " + \
#         "search DIR for libraries (multiple instances allowed)"
#   print "   -C option   " + \
#         "pass OPTION to the compiler at compile time (multiple instances " + \
#         "allowed)"
#   print "   -R option   " + \
#         "pass OPTION to the compiler at link time (multiple instances " + \
#         "allowed)"
#   sys.exit(0)

# Default option values
include_dirs = []
library_dirs = []
extra_compile_args = []
extra_link_args = []

## TODO: Use ArgumentParser.parse_known_args
# # No args?  Give usage.
# if len(sys.argv) < 2:
#   _do_usage()

# # Parse the command-line arguments, keeping what we want and letting
# # distutils have the rest.  Distutils parameters should come after
# # the target as in 'python setup.py build --prefix=/usr/local' and
# # parameters for us should appear before the target as in
# # 'python setup.py -I/usr/include build'.
# options, leftovers = getopt.getopt(sys.argv[1:], "I:L:C:R:h",
#                                    ["help"])
# for option in options:
#   if option[0] == '-I':
#     include_dirs.append(option[1])
#   if option[0] == '-L':
#     library_dirs.append(option[1])
#   if option[0] == '-C':
#     extra_compile_args.append(option[1])
#   if option[0] == '-R':
#     extra_link_args.append(option[1])
#   if option[0] == '-h':
#     _do_usage()
#
#   if option[0] == '--help':
#     _do_usage()
#
#   # All long options just get passed through
#   if option[0][:2] == '--':
#     leftovers.append(option[0])
#     leftovers.append(option[1])
# sys.argv[1:] = leftovers

from distutils.extension import Extension
import numpy as np

# Use mpi4py dist utils: https://bitbucket.org/mpi4py/mpi4py
from conf.mpidistutils import setup
#from distutils.core import setup
from distutils.spawn import find_executable
from distutils.core import Command

import subprocess
from subprocess import check_output

include_dirs.insert(0, np.get_include())
extra_compile_args.insert(0, '-Wno-uninitialized')
extra_compile_args.insert(0, '-Wno-unused-function')

m1 = Extension('adios_mpi.adios_mpi',
               sources=['adios_mpi.cpp'],
               define_macros=[],
               include_dirs = include_dirs,
               library_dirs = library_dirs,
               libraries = [],
               extra_objects = [],
               extra_compile_args = extra_compile_args,
               extra_link_args = extra_link_args)

cmd = find_executable("adios_config")
if cmd == None:
    sys.stderr.write(
        "adios_config is not installed nor found. "
        "Please install Adios or check PATH.\n")
    sys.exit(-1)

out = check_output(["adios_config", "-c"]).decode("utf-8")
for path in out.strip().split(" "):
    if path.startswith('-I'):
        m1.include_dirs.append(path.replace('-I', '', 1))

out = check_output(["adios_config", "-l"]).decode("utf-8")
for path in out.strip().split(" "):
    if path.startswith('-L'):
        m1.library_dirs.append(path.replace('-L', '', 1))
    if path.startswith('-l'):
        m1.libraries.append(path.replace('-l', '', 1))

class adios_test(Command):
    user_options = []
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import subprocess
        import sys
        errno = subprocess.call([sys.executable, 'tests/test_adios_mpi.py', 'tests/config_mpi.xml'])
        raise SystemExit(errno)

NAME = 'adios_mpi'
DESCRIPTION = 'Python Module for Adios MPI'
AUTHOR = 'Jong Choi'
AUTHOR_EMAIL = 'choij@ornl.gov'
URL = 'http://www.olcf.ornl.gov/center-projects/adios/'

import re
module_file = open("src_mpi/__init__.py").read()
metadata = dict(re.findall("__([a-z]+)__\s*=\s*'([^']+)'", module_file))
VERSION = metadata['version']

setup(name = NAME,
      version = VERSION,
      description = DESCRIPTION,
      author = AUTHOR,
      author_email = AUTHOR_EMAIL,
      url = URL,
      cmdclass={'test': adios_test},
      executables = [],
      ext_modules = [m1],
      packages=['adios_mpi', 'adios_mpi._hl'],
      package_dir = {'adios_mpi': 'src_mpi', 'adios_mpi._hl': '_hl'},
      )
